/* Created by Sumit Mehta
 * This class creates a Client Application
 */
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.border.TitledBorder;
import java.io.*;
import java.net.*;

public class Client extends JFrame {

	private JPanel p1,p2;
	private JTextArea txtReceivedData;
	private JTextField txtSendData;
	private JButton btSend;
	
	private ObjectOutputStream output;
   	private ObjectInputStream input;
   	private String message = "";
   	private Socket client;
	
	// connect to server and process messages from server
   private void runClient() 
   {
      // connect to server, get streams, process connection
      try {
         connectToServer(); // Step 1: Create a Socket to make connection
         getStreams();      // Step 2: Get the input and output streams
         processConnection(); // Step 3: Process connection
      }

      // server closed connection
      catch ( EOFException eofException ) {
         System.err.println( "Client terminated connection" );
      }

      // process problems communicating with server
      catch ( IOException ioException ) {
         ioException.printStackTrace();
      }

      finally {
         closeConnection(); // Step 4: Close connection
      }

   } // end method runClient

   // connect to server
   private void connectToServer() throws IOException {      
   
   		// create Socket to make connection to server
     	client = new Socket(InetAddress.getByName("localhost"), 12345);

      	// display connection information
      	displayMessage( "Connected to Server");
   }

   // get streams to send and receive data
   private void getStreams() throws IOException {
      
      // set up output stream for objects
      output = new ObjectOutputStream( client.getOutputStream() );      
      output.flush(); // flush output buffer to send header information

      // set up input stream for objects
      input = new ObjectInputStream( client.getInputStream());
   }

   // process connection with server
   private void processConnection() throws IOException
   {
      // enable enterField so client user can send messages

      do { // process messages sent from server

         // read message and display it
         try {
            message = ( String ) input.readObject();
            displayMessage( "\n" + message );
         }

         // catch problems reading from server
         catch ( ClassNotFoundException classNotFoundException ) {
            displayMessage( "\nUnknown object type received" );
         }

      } while ( !message.equals( "SERVER>>> TERMINATE" ) );

   } // end method processConnection

   // close streams and socket
   private void closeConnection() 
   {
      try {
         output.close();
         input.close();
         client.close();
      }
      catch( IOException ioException ) {
         ioException.printStackTrace();
      }
   }

   // send message to server
   private void sendData( String message )
   {
      // send object to server
      try {
         output.writeObject( "CLIENT>>> " + message );
         output.flush();
         displayMessage( "\nCLIENT>>> " + message );
      }

      // process problems sending object
      catch ( IOException ioException ) {
         txtReceivedData.append( "\nError writing object" );
      }
   }

   // utility method called from other threads to manipulate 
   // displayArea in the event-dispatch thread
   private void displayMessage(final String messageToDisplay) {
      
      // display message from GUI thread of execution
      SwingUtilities.invokeLater(
         new Runnable() {  // inner class to ensure GUI updates properly

            public void run() // updates displayArea
            {
               txtReceivedData.append(messageToDisplay);
               txtReceivedData.setCaretPosition(txtReceivedData.getText().length());
            }

         }  // end inner class

      ); // end call to SwingUtilities.invokeLater
   }
   
	public Client() {
	
		makeLayout();
	}
	public void makeLayout() {
	
		//initialize controls
		p1=new JPanel();
		p1.setBorder(new TitledBorder("Send Message"));
		p1.setLayout(new FlowLayout());
		
		p2=new JPanel();
		p2.setBorder(new TitledBorder("Messages"));
		p2.setLayout(new BorderLayout());
		
		txtReceivedData=new JTextArea();
		txtReceivedData.setWrapStyleWord(true);
		
		txtSendData=new JTextField(15);
		btSend=new JButton("Send To Server");
		
		p1.add(txtSendData);
		p1.add(btSend);
		
		p2.add(new JScrollPane(txtReceivedData),BorderLayout.CENTER);
		
		Container con=getContentPane();
		con.setLayout(new BorderLayout());
		
		con.add(p1,BorderLayout.NORTH);
		con.add(p2,BorderLayout.CENTER);
		
		btSend.addActionListener(
		new ActionListener() {
		
			public void actionPerformed(ActionEvent e) {
			
				sendData(txtSendData.getText());
			}
		}
		);
		
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setTitle("Client");
		setSize(350,500);
		setResizable(false);
		setVisible(true);
	}
	public static void main(String args[]) {
	
		Client client=new Client();
		client.runClient();
	}
}