/* Created by Sumit Mehta
 * This class creates a Server Application
 */
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.border.TitledBorder;
import java.net.*;
import java.io.*;

public class Server extends JFrame {

	private JPanel p1,p2;
	private JTextArea txtReceivedData;
	private JTextField txtSendData;
	private JButton btSend;
	
	private ServerSocket server;
	private Socket socket;
	private ObjectOutputStream output;
   	private ObjectInputStream input;
   	
	// set up and run server 
   public void runServer()
   {
      // set up server to receive connections; process connections
      try {

         // Step 1: Create a ServerSocket.
         server = new ServerSocket( 12345);

         while ( true ) {

            try {
               waitForConnection(); // Step 2: Wait for a connection.
               getStreams();        // Step 3: Get input & output streams.
               processConnection(); // Step 4: Process connection.
            }

            // process EOFException when client closes connection 
            catch ( EOFException eofException ) {
               System.err.println( "Server terminated connection" );
            }

            finally {
               closeConnection();   // Step 5: Close connection.
            }

         } // end while

      } // end try

      // process problems with I/O
      catch ( IOException ioException ) {
         ioException.printStackTrace();
      }

   } // end method runServer
   
   // wait for connection to arrive, then display connection info
   private void waitForConnection() throws IOException
   {
      displayMessage( "Waiting for connection\n" );
      socket = server.accept(); // allow server to accept connection            
      displayMessage( "Connection " + "received from the client: ");
   }

   // get streams to send and receive data
   private void getStreams() throws IOException
   {
      // set up output stream for objects
      output = new ObjectOutputStream( socket.getOutputStream() );
      output.flush(); // flush output buffer to send header information

      // set up input stream for objects
      input = new ObjectInputStream( socket.getInputStream() );
   }

   // process connection with client
   private void processConnection() throws IOException {
   	
   		String message = "";
   		
      	do { // process messages sent from client
			
         // read message and display it
         try {
            message = ( String ) input.readObject();
            displayMessage( "\n" + message );
         }

         // catch problems reading from client
         catch ( ClassNotFoundException classNotFoundException ) {
            displayMessage( "\nUnknown object type received" );
         }

      } while ( !message.equals( "CLIENT>>> TERMINATE" ) );

   } // end method processConnection

   // close streams and socket
   private void closeConnection() {
      
      displayMessage( "\nTerminating connection\n" );

      try {
         output.close();
         input.close();
         socket.close();
      }
      catch( IOException ioException ) {
         ioException.printStackTrace();
      }
   }

   // send message to client
   private void sendData( String message ) {
      
      // send object to client
      try {
         output.writeObject( "SERVER>>> " + message );
         output.flush();
         displayMessage( "\nSERVER>>> " + message );
      }

      // process problems sending object
      catch ( IOException ioException ) {
         txtReceivedData.append( "\nError writing object" );
      }
   }

   // utility method called from other threads to manipulate 
   // displayArea in the event-dispatch thread
   private void displayMessage( final String messageToDisplay ) {
      
      // display message from event-dispatch thread of execution
      SwingUtilities.invokeLater(
         new Runnable() {  // inner class to ensure GUI updates properly

			// updates displayArea
            public void run() {
               
               txtReceivedData.append( messageToDisplay );
               txtReceivedData.setCaretPosition(txtReceivedData.getText().length());
            }

         }  // end inner class

      ); // end call to SwingUtilities.invokeLater
   }
	public Server() {
	
		makeLayout();
	}
	private void makeLayout() {
	
		//initialize controls
		p1=new JPanel();
		p1.setBorder(new TitledBorder("Send Message"));
		p1.setLayout(new FlowLayout());
		
		p2=new JPanel();
		p2.setBorder(new TitledBorder("Messages"));
		p2.setLayout(new BorderLayout());
		
		txtReceivedData=new JTextArea();
		txtReceivedData.setWrapStyleWord(true);
		
		txtSendData=new JTextField(15);
		btSend=new JButton("Send To Client");
		
		p1.add(txtSendData);
		p1.add(btSend);
		
		p2.add(new JScrollPane(txtReceivedData),BorderLayout.CENTER);
		
		Container con=getContentPane();
		con.setLayout(new BorderLayout());
		
		con.add(p1,BorderLayout.NORTH);
		con.add(p2,BorderLayout.CENTER);
		
		btSend.addActionListener(
		new ActionListener() {
		
			public void actionPerformed(ActionEvent e) {
			
				sendData(txtSendData.getText());
			}
		}
		);
		
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setTitle("Server");
		setSize(350,500);
		setResizable(false);
		setVisible(true);
	}
	public static void main(String args[]) {
	
		Server s=new Server();
		s.runServer();
	}
}